import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.geom.AffineTransform;
import javax.imageio.*;

/**	This class contains methods for displaying a phylogenetic tree 
 *	- internal node names
 *	- leaf node names 
 *	- links between the various nodes
 *
 *	This class helps in visually identifying the edges corresponding to the various
 *	Q-matrices returned by the BH model and its extensions
 *
 *	@author	Vivek Jayaswal	1.0
 *	@Version 1.0
 */	

class DisplayTree {

	TreeMap pTree;
	String root;	

	ArrayList leafNodes;
	
	int xPosition[];			//X-coordinates
	int yPosition[];			//Y-coordinates
	
	int xIncrement = 40;		//Increment along the X-axis
	int yIncrement = 30;		//Increment along the Y-axis

	BufferedImage bufferedImage;
	Graphics2D g2D;

	DisplayTree(TreeMap phylogeneticTree, String rootNode, ArrayList leafNodesList) {
		pTree = phylogeneticTree;
		root = rootNode;
		leafNodes = leafNodesList;
	}

	public static void main(String[] args) {
		
	}

	/**	This method returns the number of edges linking the leaf node to the root
	 *	node
	 *
	 *	@param	leaf				Leaf node
	 *	
	 *	@return int					The number of edges from "leaf" to the root
	 */
	int getHopsToRoot(String leaf) {

		int hopCount = 0;	//default hop count value
		NodeStructure ns;		
		String parent;
		String currentNode = leaf;
		int leftOrRightNode;		

		do{
			hopCount++;			
			
			ns = (NodeStructure) pTree.get(currentNode);
			parent = ns.getParentNode();					
			
			/*	Determine whether the currentNode is the left or right child node
			 *	of the parent. If the currentNode is the left child node, add 1
			 *	to the left node counter of the parent, else add 1 to the right
			 *	node counter
			 */
			ns = (NodeStructure) pTree.get(parent);
			leftOrRightNode = ns.getLeftOrRightChildNode(currentNode);
			
			if(leftOrRightNode == 0) {
				ns.incrLeftNodeCount();
			}
			else {
				ns.incrRightNodeCount();
			} 
			
			pTree.put(parent, ns);
			
			currentNode = parent;
			
		} while(!parent.equals(root)); // root node reached
	
		return hopCount;
	}

	/**	This method is used to determine the maximum distance from the root
	 *
	 *	@return	int			Maximum number of edges
	 */
	int getMaxHopCount() {
		
		int maxHopCount = 0;
		int currentHopCount;
	
		for(int i=leafNodes.size()-1; i>=0; i--) {
			currentHopCount = getHopsToRoot((String)leafNodes.get(i));
			
			if(currentHopCount > maxHopCount) maxHopCount = currentHopCount;
		}

		return maxHopCount;
	}

	/**	This method is used to initialize the 2-D grid on which the internal nodes
	 *	and leaf nodes are plotted
	 *
	 *	@param	maxHops				Maximum number of hops from root to a leaf node
	 */
	void initializeGrid(int maxHops) {

		int leafNodeCount = leafNodes.size();

		xPosition = new int[leafNodeCount];
		yPosition = new int[maxHops];
		
		xPosition[0] = 40;
		yPosition[0] = 40;

		for(int i=1; i<leafNodeCount; i++) {
			xPosition[i] = xPosition[i-1] + xIncrement;
			//System.out.println("X[" + i + "] = " + xPosition[i]);
		}

		for(int i=1; i<maxHops; i++) {
			yPosition[i] = yPosition[i-1] + yIncrement;
			//System.out.println("Y[" + i + "] = " + yPosition[i]);
		}
		
	}

	/**	This method is used to plot the "root" and the internal node connected to
	 *	it. It then calls the method for plotting the rest of the tree
	 */
	void setPositionRootToInt() {
					
		//Set the position of the internal node linked to the root node
		NodeStructure ns = (NodeStructure) pTree.get(root);
		String intNode = ns.getChildNode1();

		ns = (NodeStructure) pTree.get(intNode);
		int leftNodeCount = ns.getLeftNodeCount();

		String label = getNodeLabel(intNode, 0);
		
		g2D.setColor(Color.red);
		writeNodeValue(leftNodeCount-1, 0, 0, label);
		
		//Plot the root node
		label = getNodeLabel(root, 1);
		writeNodeValue(leftNodeCount-1, -1, 0, label);
		
		g2D.setColor(Color.black);
		
		plotLine(leftNodeCount-1, -1, leftNodeCount-1, 0);

		//Call the traverseTree method to generate the rest of the tree structure	
		traverseTree(intNode, leftNodeCount-1, 0); 
	}

	/**	This method is used to determine a particular node's label. If the node is
	 *	an internal node, the label is of the form "XXX" e.g. an internal node
	 *	1-Node will be labelled XX1. If the node is a leaf node, the node name
	 *	(excluding "-Node") becomes the label
	 *	
	 *	@param	node			Node name
	 *	@param	nodeType		0 -> Internal node
	 *							1 -> Leaf node
	 *	
	 *	@return String			Node label
	 */
	String getNodeLabel(String node, int nodeType) {
		
		String label = "";
		int index = node.indexOf('-');
		
		if(nodeType==0) {
			//Internal node to be labelled as XXX						
			label = node.substring(0,index);

			switch(label.length()) {
				case 1: label = "XX" + label; break;
				case 2: label = "X" + label;
			}			
		}
		else {
			//Leaf node to be labelled using the first 5 characters of taxa name
			label = node.substring(0, index);
		}

		return label;
	} 	

	/**	This method is used to plot the node lable at the user-specified position.
	 *	Internal nodes are plotted horizontally while leaf nodes are plotted
	 *	vertically
	 *
	 *	@param	x_pos				X-coordinate of the label
	 *	@param	y_pos				Y-coordinate of the label
	 *	@param	leafOrInternal		0 -> Internal node
	 *								1 -> Leaf node
	 *	@param	label				Node label
	 */
	void writeNodeValue(int x_pos, int y_pos, int leafOrInternal, String label) {
		
/*
		System.out.println("X = " + x_pos
					+ " Y = " + y_pos
					+ " Type = " + leafOrInternal 
					+ " Label = " + label);
*/
		AffineTransform at = new AffineTransform();

		int x_value = xPosition[x_pos];
		int y_value;
		
		if(y_pos >= 0) y_value = yPosition[y_pos];
		else y_value = yPosition[0] - 20;
		
		if(leafOrInternal==0) {
			//Inernal node labels are displayed horizontally
			g2D.setTransform(at);
			g2D.drawString(label, x_value-15, y_value);
		}
		else {
			//Leaf node labels are displayed horizontally
		    	at.setToRotation(Math.PI/2.0, x_value, y_value);
			g2D.setTransform(at);	
			g2D.drawString(label, x_value, y_value);			
		}

		//Re-set the plotting direction
		at = new AffineTransform();
		g2D.setTransform(at);
	}	

	/**	This is recursive method for plotting the tree
	 *
	 *	@param	currentNode				Current tree node
	 *	@param	x_Current				Current X-coordinate
	 *	@param	y_Current				Current Y-coordinate
	 */	
	void traverseTree(String currentNode, int x_Current, int y_Current) {

		NodeStructure ns = (NodeStructure) pTree.get(currentNode);
		NodeStructure nsTemp;

		int leftNodeCount = ns.getLeftNodeCount();
		int rightNodeCount = ns.getRightNodeCount();

		int tempNodeCount;
		int tempRightNodeCount;
		int tempLeftNodeCount;

		String childNode;
		String leftChildNode;
		String rightChildNode;	
		String nodeLabel;
		
		//Determine whether (leftNodeCount, rightNodeCount) = (1, 2) or (2, 1) or (2, 2)
		if((leftNodeCount <= 2) && (rightNodeCount <=2)) {
			plotNodes(currentNode, x_Current, y_Current, leftNodeCount, rightNodeCount);
		} 

		//Determine whether (leftNodeCount, rightNodeCount) > (2,2)	
		if((leftNodeCount > 2) && (rightNodeCount > 2)) {
			
			/*	Obtain the location of the left child node of "currentNode" and plot it
			 *	x-coordinate of left_child_node = x_Current - size of right node of left_child_node
			 */
			childNode = ns.getChildNode1();
			nodeLabel = getNodeLabel(childNode, 0); 
			
			nsTemp = (NodeStructure) pTree.get(childNode);
			tempRightNodeCount = nsTemp.getRightNodeCount();

			writeNodeValue(x_Current - tempRightNodeCount, y_Current + 1, 0, nodeLabel); 
	
			plotLine(x_Current, y_Current, x_Current-tempRightNodeCount, y_Current+1);

			traverseTree(childNode, x_Current - tempRightNodeCount, y_Current + 1); 
			
			/*	Obtain the location of the right child node of "currentNode" and plot it
			 *	x-coordinate of right_child_node = x_Current + size of left node of right_child_node
			 */
			childNode = ns.getChildNode2();
			nodeLabel = getNodeLabel(childNode, 0); 

			nsTemp = (NodeStructure) pTree.get(childNode);
			tempLeftNodeCount = nsTemp.getLeftNodeCount();

			writeNodeValue(x_Current + tempLeftNodeCount, y_Current + 1, 0, nodeLabel); 

			plotLine(x_Current, y_Current, x_Current+tempLeftNodeCount, y_Current+1);

			traverseTree(childNode, x_Current + tempLeftNodeCount, y_Current + 1); 
		} // (leftNodeCount, rightNodeCount) > (2,2)	

		//Determine whether (leftNodeCount, rightNodeCount) = (1,X) where X >=2
		if((leftNodeCount == 1) && (rightNodeCount > 2)) {

			/*	Obtain the structure
			 *				(NODE)
			 *				  |	    \
			 *				  L1    XXX
			 */

			//Obtain the left node (leaf node)
			leftChildNode = ns.getChildNode1();			
			nodeLabel = getNodeLabel(leftChildNode, 1);
			writeNodeValue(x_Current, y_Current + 1, 1, nodeLabel); 
			
			plotLine(x_Current, y_Current, x_Current, y_Current+1);

			//Obtain the right node (internal node)
			rightChildNode = ns.getChildNode2();
			ns = (NodeStructure) pTree.get(rightChildNode);
			tempNodeCount = ns.getLeftNodeCount();

			nodeLabel = getNodeLabel(rightChildNode, 0);
			writeNodeValue(x_Current + tempNodeCount, y_Current + 1, 0, nodeLabel); 
			
			plotLine(x_Current, y_Current, x_Current+tempNodeCount, y_Current+1);

			traverseTree(rightChildNode, x_Current + tempNodeCount, y_Current + 1); 		
		}
		
		//Determine whether (leftNodeCount, rightNodeCount) = (X,1) where X >=2
		if((leftNodeCount > 2) && (rightNodeCount == 1)) {

			/*	Obtain the structure
			 *				(NODE)
			 *	  		     /   \
			 *			   XXX 	  L1		
			 */

			//Obtain the right node (leaf node)
			rightChildNode = ns.getChildNode2();			
			nodeLabel = getNodeLabel(rightChildNode, 1);
			writeNodeValue(x_Current+1, y_Current + 1, 1, nodeLabel); 
			
			plotLine(x_Current, y_Current, x_Current+1, y_Current+1);

			//Obtain the left node (internal node)
			leftChildNode = ns.getChildNode1();
			ns = (NodeStructure) pTree.get(leftChildNode);
			tempNodeCount = ns.getRightNodeCount();

			nodeLabel = getNodeLabel(leftChildNode, 0);
			writeNodeValue(x_Current - tempNodeCount, y_Current + 1, 0, nodeLabel); 
			
			plotLine(x_Current, y_Current, x_Current-tempNodeCount, y_Current+1);

			traverseTree(leftChildNode, x_Current - tempNodeCount, y_Current + 1); 		
		}		

		//Determine whether (leftNodeCount, rightNodeCount) = (2,X) where X >=2
		if((leftNodeCount == 2) && (rightNodeCount > 2)) {
			
			/*	Obtain the structure
			 *				(NODE)
			 *			   /	   \
			 *			INT			XXX
			 *	         |   \
			 *			 L1   L2	
			 */
			leftChildNode = ns.getChildNode1();
			nodeLabel = getNodeLabel(leftChildNode, 0);
			writeNodeValue(x_Current-1, y_Current + 1, 0, nodeLabel); 
			
			plotLine(x_Current, y_Current, x_Current-1, y_Current+1);
		
			nsTemp = (NodeStructure) pTree.get(leftChildNode);
			childNode = nsTemp.getChildNode1();		//LEFT child node
			nodeLabel = getNodeLabel(childNode, 1);
			writeNodeValue(x_Current-1, y_Current + 2, 1, nodeLabel);	 	
			
			plotLine(x_Current-1, y_Current+1, x_Current-1, y_Current+2);

			childNode = nsTemp.getChildNode2();		//RIGHT child node
			nodeLabel = getNodeLabel(childNode, 1);
			writeNodeValue(x_Current, y_Current + 2, 1, nodeLabel);	 
			
			plotLine(x_Current-1, y_Current+1, x_Current, y_Current+2);

			//Obtain the right node (internal node)
			rightChildNode = ns.getChildNode2();
			ns = (NodeStructure) pTree.get(rightChildNode);
			tempNodeCount = ns.getLeftNodeCount();

			nodeLabel = getNodeLabel(rightChildNode, 0);
			writeNodeValue(x_Current + tempNodeCount, y_Current + 1, 0, nodeLabel); 
			
			plotLine(x_Current, y_Current, x_Current+tempNodeCount, y_Current+1);

			traverseTree(rightChildNode, x_Current + tempNodeCount, y_Current + 1); 	
		}
		
		//Determine whether (leftNodeCount, rightNodeCount) = (X,2) where X >=2
		if((leftNodeCount > 2) && (rightNodeCount == 2)) {
			
			/*	Obtain the structure
			 *				(NODE)
			 *			   /	 \
			 *			XXX		 INT
			 *					  |	\
			 *					 L1	 L2
			 */
			rightChildNode = ns.getChildNode2();
			nodeLabel = getNodeLabel(rightChildNode, 0);
			writeNodeValue(x_Current+1, y_Current + 1, 0, nodeLabel); 
			
			plotLine(x_Current, y_Current, x_Current+1, y_Current+1);
		
			nsTemp = (NodeStructure) pTree.get(rightChildNode);
			childNode = nsTemp.getChildNode1();		//LEFT child node
			nodeLabel = getNodeLabel(childNode, 1);
			writeNodeValue(x_Current+1, y_Current + 2, 1, nodeLabel);	 	
			
			plotLine(x_Current+1, y_Current+1, x_Current+1, y_Current+2);

			childNode = nsTemp.getChildNode2();		//RIGHT child node
			nodeLabel = getNodeLabel(childNode, 1);
			writeNodeValue(x_Current+2, y_Current + 2, 1, nodeLabel);	 
			
			plotLine(x_Current+1, y_Current+1, x_Current+2, y_Current+2);

			//Obtain the left node (internal node)
			leftChildNode = ns.getChildNode1();
			ns = (NodeStructure) pTree.get(leftChildNode);
			tempNodeCount = ns.getRightNodeCount();

			nodeLabel = getNodeLabel(leftChildNode, 0);
			writeNodeValue(x_Current - tempNodeCount, y_Current + 1, 0, nodeLabel); 
			
			plotLine(x_Current, y_Current, x_Current-tempNodeCount, y_Current+1);

			traverseTree(leftChildNode, x_Current - tempNodeCount, y_Current + 1); 	
		}

	}
		
	/**	This method is used for plotting a sub-tree of the form (1,2), (2,1) or (2,2)
	 *
	 *	@param	currentNode					Current tree node
	 *	@param	x_value						Current X-coordinate
	 *	@param	y_value						Current Y-coordinate
	 *	@param	leftCount					Number of leaf nodes to the left of "currentNode"
	 *	@param	rightCount					Number of leaf nodes to the right of "currentNode"	
	 */
	void plotNodes(String currentNode, int x_value, int y_value, int leftCount, int rightCount) {

		NodeStructure ns;		

		String intNode1;
		String intNode2;
		String leafNode1;
		String leafNode2;
		String nodeLabel;

		int x_Current;
		int y_Current;

		/*	The structure to be plotted is
		 *				X
		 *	  			|  \
		 *				Y1	Y2
		 * 					 |	\
		 *					Y3	Y4
		 */	
		
		if((leftCount == 1) && (rightCount == 2)) {
		
			//Plot the left child node of "currentNode" which is a leaf node

			ns = (NodeStructure) pTree.get(currentNode);
			leafNode1 = ns.getChildNode1();
			nodeLabel = getNodeLabel(leafNode1, 1); 
			writeNodeValue(x_value, y_value+1, 1, nodeLabel); 

			plotLine(x_value, y_value, x_value, y_value+1);
			
			/*	Plot the right child node of "currentNode". This happens
			 *	to be an internal node to which two leaf nodes are connected
			 */
			intNode2 = ns.getChildNode2();
			nodeLabel = getNodeLabel(intNode2, 0); 
			writeNodeValue(x_value+1, y_value+1, 0, nodeLabel);

			plotLine(x_value, y_value, x_value+1, y_value+1);
		
			x_Current = x_value+1;
			y_Current = y_value+1;

			ns = (NodeStructure) pTree.get(intNode2);
			leafNode1 = ns.getChildNode1();
			nodeLabel = getNodeLabel(leafNode1, 1); 
			writeNodeValue(x_Current, y_Current+1, 1, nodeLabel); 
			
			plotLine(x_Current, y_Current, x_Current, y_Current+1);

			leafNode2 = ns.getChildNode2();
			nodeLabel = getNodeLabel(leafNode2, 1); 
			writeNodeValue(x_Current+1, y_Current+1, 1, nodeLabel);
 
			plotLine(x_Current, y_Current, x_Current+1, y_Current+1);

		} //leftCount = 1 and rightCount=2

		/*	The structure to be plotted is
		 *				X
		 *	    		|  \
		 *				Y1	Y2
		 * 			 /	|	
		 *			Y3	Y4
		 */	

		if((leftCount == 2) && (rightCount == 1)) {

			//Plot the right child node of "currentNode" which is a leaf node

			ns = (NodeStructure) pTree.get(currentNode);
			leafNode2 = ns.getChildNode2();
			nodeLabel = getNodeLabel(leafNode2, 1); 
			writeNodeValue(x_value+1, y_value+1, 1, nodeLabel); 

			plotLine(x_value, y_value, x_value+1, y_value+1);
			
			/*	Plot the left child node of "currentNode". This happens
			 *	to be an internal node to which two leaf nodes are connected
			 */
			intNode1 = ns.getChildNode1();
			nodeLabel = getNodeLabel(intNode1, 0); 
			writeNodeValue(x_value, y_value+1, 0, nodeLabel);

			plotLine(x_value, y_value, x_value, y_value+1);
		
			x_Current = x_value;
			y_Current = y_value+1;

			ns = (NodeStructure) pTree.get(intNode1);
			leafNode1 = ns.getChildNode1();
			nodeLabel = getNodeLabel(leafNode1, 1); 
			writeNodeValue(x_Current-1, y_Current+1, 1, nodeLabel); 

			plotLine(x_Current, y_Current, x_Current-1, y_Current+1);

			leafNode2 = ns.getChildNode2();
			nodeLabel = getNodeLabel(leafNode2, 1); 
			writeNodeValue(x_Current, y_Current+1, 1, nodeLabel); 			

			plotLine(x_Current, y_Current, x_Current, y_Current+1);

		} //leftCount = 2 and rightCount=1

		/*	The structure to be plotted is
		 *				X
		 *	         /	   \
		 *			Y1		Y2
		 * 			|  \	 | \
		 *			Y3	Y4	Y5	Y6
		 */	

		if((leftCount == 2) && (rightCount == 2)) {
			
			ns = (NodeStructure) pTree.get(currentNode);	

			/*	Plot the left child node of "currentNode". This happens
			 *	to be an internal node to which two leaf nodes are connected
			 */
			intNode1 = ns.getChildNode1();
			nodeLabel = getNodeLabel(intNode1, 0); 
			writeNodeValue(x_value-1, y_value+1, 0, nodeLabel);

			plotLine(x_value, y_value, x_value-1, y_value+1);
		
			x_Current = x_value-1;
			y_Current = y_value+1;

			ns = (NodeStructure) pTree.get(intNode1);
			leafNode1 = ns.getChildNode1();
			nodeLabel = getNodeLabel(leafNode1, 1); 
			writeNodeValue(x_Current, y_Current+1, 1, nodeLabel); 

			plotLine(x_Current, y_Current, x_Current, y_Current+1);

			leafNode2 = ns.getChildNode2();
			nodeLabel = getNodeLabel(leafNode2, 1); 
			writeNodeValue(x_Current+1, y_Current+1, 1, nodeLabel); 	
			
			plotLine(x_Current, y_Current, x_Current+1, y_Current+1);

			/*	Plot the right child node of "currentNode". This happens
			 *	to be an internal node to which two leaf nodes are connected
			 */
			ns = (NodeStructure) pTree.get(currentNode);

			intNode2 = ns.getChildNode2();
			nodeLabel = getNodeLabel(intNode2, 0); 
			writeNodeValue(x_value+1, y_value+1, 0, nodeLabel);

			plotLine(x_value, y_value, x_value+1, y_value+1);
		
			x_Current = x_value+1;
			y_Current = y_value+1;

			ns = (NodeStructure) pTree.get(intNode2);
			leafNode1 = ns.getChildNode1();
			nodeLabel = getNodeLabel(leafNode1, 1); 
			writeNodeValue(x_Current, y_Current+1, 1, nodeLabel); 

			plotLine(x_Current, y_Current, x_Current, y_Current+1);

			leafNode2 = ns.getChildNode2();
			nodeLabel = getNodeLabel(leafNode2, 1); 
			writeNodeValue(x_Current+1, y_Current+1, 1, nodeLabel); 
		
			plotLine(x_Current, y_Current, x_Current+1, y_Current+1);

		} //leftCount = 2 and rightCount=2
		
	}

	/**	This method is used to plot a line linking a pair of nodes
	 *	
	 *	@param x1				X-coordinate of point 1 (parent node)
	 *	@param y1				Y-coordinate of point 1 (parent node)
	 *	@param x1				X-coordinate of point 2 (child node)
	 *	@param y1				Y-coordinate of point 2 (child node)
	 */
	void plotLine(int x1, int y1, int x2, int y2) {

/*		System.out.println("Plot "
					+ " X1 = " + x1 + " Position = " + xPosition[x1]
					+ " Y1 = " + y1 + " Position = " + yPosition[y1]
					+ " X2 = " + x2 + " Position = " + xPosition[x2]
					+ " Y2 = " + y2 + " Position = " + yPosition[y2]
					);
*/
		if(y1 == -1) {
			g2D.drawLine(xPosition[x1], yPosition[0]-20
					, xPosition[x2], yPosition[y2]-10);						
		}
		else  {
			g2D.drawLine(xPosition[x1], yPosition[y1]
					, xPosition[x2], yPosition[y2]-10);			
		}
		
	}	

	/**	This method is used for displaying the (x,y) coordinates of the various
	 *	nodes - internal and leaf.
	 */
	void showTreeSructure() {
		
		Set set = pTree.entrySet();
		Iterator itr = set.iterator();
		
		while(itr.hasNext()) {
			Map.Entry me = (Map.Entry)itr.next();
			
			NodeStructure node = (NodeStructure) me.getValue();
			String name = (String)me.getKey();
                        
/*                   System.out.println(" ");
                   System.out.println("Node Name: " + 	name);
	   	       System.out.println("Left Node Count: " +  node.getLeftNodeCount());
			 System.out.println("Right Node Count: " + node.getRightNodeCount());
*/		}				
	}

	/**	This is the main method for displaying the phylogenetic tree. It is called
	 *	from the class NewickTreeTraversal
	 *
	 *	@param	filePath				File path of the picture file
	 */
	void createJPGFile(String filePath) {

		int maxHops = getMaxHopCount();	//Consider hops to the internal node linked to the root node
		//System.out.println("Maximum number of hops = " + maxHops); 

		initializeGrid(maxHops);

		int leafNodeCount = leafNodes.size();
		int xAxisSize = xPosition[leafNodeCount-1] + xIncrement;
		int yAxisSize = yPosition[maxHops-1] + 100;
		
		bufferedImage = new BufferedImage(xAxisSize, yAxisSize, BufferedImage.TYPE_INT_RGB);
        g2D = bufferedImage.createGraphics();   
		
		//Create a white background 
      	g2D.setColor(Color.white);
		g2D.fillRect(0, 0, xAxisSize, yAxisSize);
		
		g2D.setColor(Color.black);

		setPositionRootToInt();
	    
		// Write generated image to a file
            try {
            	// Save as JPEG
            	File file = new File(filePath);
            	ImageIO.write(bufferedImage, "jpg", file);

        	} catch (IOException e) {
            	System.out.println("Error occured " + e);
        	}
	
    }


}